/*=========================================================================
| Beagle Interface Library
|--------------------------------------------------------------------------
| Copyright (c) 2004-2007 Total Phase, Inc.
| All rights reserved.
| www.totalphase.com
|
| Redistribution and use in source and binary forms, with or without
| modification, are permitted provided that the following conditions
| are met:
|
| - Redistributions of source code must retain the above copyright
|   notice, this list of conditions and the following disclaimer.
|
| - Redistributions in binary form must reproduce the above copyright
|   notice, this list of conditions and the following disclaimer in the
|   documentation and/or other materials provided with the distribution.
|
| - Neither the name of Total Phase, Inc. nor the names of its
|   contributors may be used to endorse or promote products derived from
|   this software without specific prior written permission.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
|--------------------------------------------------------------------------
| To access Total Phase Beagle devices through the API:
|
| 1) Use one of the following shared objects:
|      beagle.so        --  Linux shared object
|          or
|      beagle.dll       --  Windows dynamic link library
|
| 2) Along with one of the following language modules:
|      beagle.c/h       --  C/C++ API header file and interface module
|      beagle_py.py     --  Python API
|      beagle.bas       --  Visual Basic 6 API
|      beagle.cs        --  C# .NET source
|      beagle_net.dll   --  Compiled .NET binding
 ========================================================================*/


#ifndef __beagle_h__
#define __beagle_h__

#ifdef __cplusplus
extern "C" {
#endif


/*=========================================================================
| TYPEDEFS
 ========================================================================*/
#ifndef TOTALPHASE_DATA_TYPES
#define TOTALPHASE_DATA_TYPES

#ifndef _MSC_VER
/* C99-compliant compilers (GCC) */
#include <stdint.h>
typedef uint8_t   u08;
typedef uint16_t  u16;
typedef uint32_t  u32;
typedef uint64_t  u64;
typedef int8_t    s08;
typedef int16_t   s16;
typedef int32_t   s32;
typedef int64_t   s64;

#else
/* Microsoft compilers (Visual C++) */
typedef unsigned __int8   u08;
typedef unsigned __int16  u16;
typedef unsigned __int32  u32;
typedef unsigned __int64  u64;
typedef signed   __int8   s08;
typedef signed   __int16  s16;
typedef signed   __int32  s32;
typedef signed   __int64  s64;

#endif /* __MSC_VER */

#endif /* TOTALPHASE_DATA_TYPES */


/*=========================================================================
| DEBUG
 ========================================================================*/
/* Set the following macro to '1' for debugging */
#define BG_DEBUG 0


/*=========================================================================
| VERSION
 ========================================================================*/
#define BG_HEADER_VERSION  0x0200   /* v2.00 */


/*=========================================================================
| STATUS CODES
 ========================================================================*/
/*
 * All API functions return an integer which is the result of the
 * transaction, or a status code if negative.  The status codes are
 * defined as follows:
 */
enum BeagleStatus {
    /* General codes (0 to -99) */
    BG_OK                      =    0,
    BG_UNABLE_TO_LOAD_LIBRARY  =   -1,
    BG_UNABLE_TO_LOAD_DRIVER   =   -2,
    BG_UNABLE_TO_LOAD_FUNCTION =   -3,
    BG_INCOMPATIBLE_LIBRARY    =   -4,
    BG_INCOMPATIBLE_DEVICE     =   -5,
    BG_INCOMPATIBLE_DRIVER     =   -6,
    BG_COMMUNICATION_ERROR     =   -7,
    BG_UNABLE_TO_OPEN          =   -8,
    BG_UNABLE_TO_CLOSE         =   -9,
    BG_INVALID_HANDLE          =  -10,
    BG_CONFIG_ERROR            =  -11,
    BG_UNKNOWN_PROTOCOL        =  -12,
    BG_STILL_ACTIVE            =  -13,
    BG_FUNCTION_NOT_AVAILABLE  =  -14,

    /* COMMTEST codes (-100 to -199) */
    BG_COMMTEST_NOT_AVAILABLE  = -100,
    BG_COMMTEST_NOT_ENABLED    = -101,

    /* I2C codes (-200 to -299) */
    BG_I2C_NOT_AVAILABLE       = -200,
    BG_I2C_NOT_ENABLED         = -201,

    /* SPI codes (-300 to -399) */
    BG_SPI_NOT_AVAILABLE       = -300,
    BG_SPI_NOT_ENABLED         = -301,

    /* USB codes (-400 to -499) */
    BG_USB_NOT_AVAILABLE       = -400,
    BG_USB_NOT_ENABLED         = -401,

    /* MDIO codes (-500 to -599) */
    BG_MDIO_NOT_AVAILABLE      = -500,
    BG_MDIO_NOT_ENABLED        = -501,
    BG_MDIO_BAD_TURNAROUND     = -502
};
#ifndef __cplusplus
typedef enum BeagleStatus BeagleStatus;
#endif


/*=========================================================================
| GENERAL TYPE DEFINITIONS
 ========================================================================*/
/* Beagle handle type definition */
typedef int Beagle;

/*
 * Beagle version matrix.
 * 
 * This matrix describes the various version dependencies
 * of Beagle components.  It can be used to determine
 * which component caused an incompatibility error.
 * 
 * All version numbers are of the format:
 *   (major << 8) | minor
 * 
 * ex. v1.20 would be encoded as:  0x0114
 */
struct BeagleVersion {
    /* Software and hardware versions. */
    u16 software;
    u16 hardware;

    /*
     * Hardware revisions that are compatible with this software version.
     * The top 16 bits gives the maximum accepted hw revision.
     * The lower 16 bits gives the minimum accepted hw revision.
     */
    u32 hw_revs_for_sw;

    /*
     * Driver revisions that are compatible with this software version.
     * The top 16 bits gives the maximum accepted driver revision.
     * The lower 16 bits gives the minimum accepted driver revision.
     * This version checking is currently only pertinent for WIN32
     * platforms.
     */
    u32 drv_revs_for_sw;

    /* Software requires that the API interface must be >= this version. */
    u16 api_req_by_sw;
};
#ifndef __cplusplus
typedef struct BeagleVersion BeagleVersion;
#endif


/*=========================================================================
| GENERAL API
 ========================================================================*/
/*
 * Get a list of ports to which Beagle devices are attached.
 * 
 * nelem   = maximum number of elements to return
 * devices = array into which the port numbers are returned
 * 
 * Each element of the array is written with the port number.
 * Devices that are in-use are ORed with BG_PORT_NOT_FREE
 * (0x8000).
 *
 * ex.  devices are attached to ports 0, 1, 2
 *      ports 0 and 2 are available, and port 1 is in-use.
 *      array => 0x0000, 0x8001, 0x0002
 * 
 * If the array is NULL, it is not filled with any values.
 * If there are more devices than the array size, only the
 * first nmemb port numbers will be written into the array.
 *
 * Returns the number of devices found, regardless of the
 * array size.
 */
#define BG_PORT_NOT_FREE 0x8000
int bg_find_devices (
    int   nelem,
    u16 * devices
);


/*
 * Get a list of ports to which Beagle devices are attached
 *
 * This function is the same as bg_find_devices() except that
 * it returns the unique IDs of each Beagle device.  The IDs
 * are guaranteed to be non-zero if valid.
 *
 * The IDs are the unsigned integer representation of the 10-digit
 * serial numbers.
 */
int bg_find_devices_ext (
    int   nelem,
    u16 * devices,
    u32 * unique_ids
);


/*
 * Open the Beagle port.
 * 
 * The port number is a zero-indexed integer.
 *
 * The port number is the same as that obtained from the
 * bg_find_devices() function above.
 * 
 * Returns an Beagle handle, which is guaranteed to be
 * greater than zero if it is valid.
 * 
 * This function is recommended for use in simple applications
 * where extended information is not required.  For more complex
 * applications, the use of bg_open_ext() is recommended.
 */
Beagle bg_open (
    int port_number
);


/*
 * Open the Beagle port, returning extended information
 * in the supplied structure.  Behavior is otherwise identical
 * to bg_open() above.  If 0 is passed as the pointer to the
 * structure, this function is exactly equivalent to bg_open().
 * 
 * The structure is zeroed before the open is attempted.
 * It is filled with whatever information is available.
 * 
 * For example, if the hardware version is not filled, then
 * the device could not be queried for its version number.
 * 
 * This function is recommended for use in complex applications
 * where extended information is required.  For more simple
 * applications, the use of bg_open() is recommended.
 */
struct BeagleExt {
    /* Version matrix */
    BeagleVersion version;

    /* Features of this device. */
    int           features;
};
#ifndef __cplusplus
typedef struct BeagleExt BeagleExt;
#endif

Beagle bg_open_ext (
    int         port_number,
    BeagleExt * bg_ext
);


/* Close the Beagle port. */
int bg_close (
    Beagle beagle
);


/*
 * Return the port for this Beagle handle.
 * 
 * The port number is a zero-indexed integer.
 */
int bg_port (
    Beagle beagle
);


/*
 * Return the device features as a bit-mask of values, or
 * an error code if the handle is not valid.
 */
#define BG_FEATURE_NONE 0x00000000
#define BG_FEATURE_I2C 0x00000001
#define BG_FEATURE_SPI 0x00000002
#define BG_FEATURE_USB 0x00000004
#define BG_FEATURE_MDIO 0x00000008
int bg_features (
    Beagle beagle
);


int bg_unique_id_to_features (
    u32 unique_id
);


/*
 * Return the unique ID for this Beagle adapter.
 * IDs are guaranteed to be non-zero if valid.
 * The ID is the unsigned integer representation of the
 * 10-digit serial number.
 */
u32 bg_unique_id (
    Beagle beagle
);


/*
 * Return the status string for the given status code.
 * If the code is not valid or the library function cannot
 * be loaded, return a NULL string.
 */
const char * bg_status_string (
    int status
);


/*
 * Return the version matrix for the device attached to the
 * given handle.  If the handle is 0 or invalid, only the
 * software and required api versions are set.
 */
int bg_version (
    Beagle          beagle,
    BeagleVersion * version
);


/*
 * Set the capture latency to the specified number of milliseconds.
 * This number determines the minimum time that a read call will
 * block if there is no available data.  Lower times result in
 * faster turnaround at the expense of reduced buffering.  Setting
 * this parameter too low can cause packets to be dropped.
 */
int bg_latency (
    Beagle beagle,
    u32    milliseconds
);


/*
 * Set the capture timeout to the specified number of milliseconds.
 * If any read call has a longer idle than this value, that call
 * will return with a count of 0 bytes.
 */
int bg_timeout (
    Beagle beagle,
    u32    milliseconds
);


/*
 * Sleep for the specified number of milliseconds
 * Accuracy depends on the operating system scheduler
 * Returns the number of milliseconds slept
 */
u32 bg_sleep_ms (
    u32 milliseconds
);


/* Configure the target power pin. */
#define BG_TARGET_POWER_OFF 0x00
#define BG_TARGET_POWER_ON 0x01
#define BG_TARGET_POWER_QUERY 0x80
int bg_target_power (
    Beagle beagle,
    u08    power_flag
);


#define BG_HOST_IFCE_FULL_SPEED 0x00
#define BG_HOST_IFCE_HIGH_SPEED 0x01
int bg_host_ifce_speed (
    Beagle beagle
);



/*=========================================================================
| BENCHMARK API
 ========================================================================*/
int bg_buffers_avail (
    Beagle beagle
);


int bg_buffers_info (
    Beagle beagle,
    int *  max_buffers,
    int *  buffer_size
);


/* Benchmark the speed of the host to Beagle interface */
int bg_commtest (
    Beagle beagle,
    int    num_samples,
    int    delay_count
);



/*=========================================================================
| MONITORING API
 ========================================================================*/
/* Protocol codes */
enum BeagleProtocol {
    BG_PROTOCOL_NONE     = 0,
    BG_PROTOCOL_COMMTEST = 1,
    BG_PROTOCOL_USB      = 2,
    BG_PROTOCOL_I2C      = 3,
    BG_PROTOCOL_SPI      = 4,
    BG_PROTOCOL_MDIO     = 5
};
#ifndef __cplusplus
typedef enum BeagleProtocol BeagleProtocol;
#endif

/* Common Beagle read status codes */
#define BG_READ_OK 0x00000000
#define BG_READ_TIMEOUT 0x00000100
#define BG_READ_ERR_MIDDLE_OF_PACKET 0x00000200
#define BG_READ_ERR_SHORT_BUFFER 0x00000400
#define BG_READ_ERR_PARTIAL_LAST_BYTE 0x00000800
/* Enable the Beagle monitor */
int bg_enable (
    Beagle         beagle,
    BeagleProtocol protocol
);


/* Disable the Beagle monitor */
int bg_disable (
    Beagle beagle
);


/* Set the sample rate in kilohertz. */
int bg_samplerate (
    Beagle beagle,
    int    samplerate_khz
);


/*
 * Get the number of bits for the given number of bytes in the
 * given protocol.
 * Use this to determine how large a bit_timing array to allocate
 * for bg_*_read_bit_timing functions.
 */
int bg_bit_timing_size (
    BeagleProtocol protocol,
    int            num_data_bytes
);



/*=========================================================================
| I2C API
 ========================================================================*/
/* Configure the I2C pullup resistors. */
#define BG_I2C_PULLUP_OFF 0x00
#define BG_I2C_PULLUP_ON 0x01
#define BG_I2C_PULLUP_QUERY 0x80
int bg_i2c_pullup (
    Beagle beagle,
    u08    pullup_flag
);


#define BG_I2C_MONITOR_DATA 0x00ff
#define BG_I2C_MONITOR_NACK 0x0100
#define BG_READ_I2C_NO_STOP 0x00010000
int bg_i2c_read (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u16 *  data_in
);


int bg_i2c_read_data_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u16 *  data_in,
    int    max_timing,
    u32 *  data_timing
);


int bg_i2c_read_bit_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u16 *  data_in,
    int    max_timing,
    u32 *  bit_timing
);



/*=========================================================================
| SPI API
 ========================================================================*/
enum BeagleSpiSSPolarity {
    BG_SPI_SS_ACTIVE_LOW  = 0,
    BG_SPI_SS_ACTIVE_HIGH = 1
};
#ifndef __cplusplus
typedef enum BeagleSpiSSPolarity BeagleSpiSSPolarity;
#endif

enum BeagleSpiSckSamplingEdge {
    BG_SPI_SCK_SAMPLING_EDGE_RISING  = 0,
    BG_SPI_SCK_SAMPLING_EDGE_FALLING = 1
};
#ifndef __cplusplus
typedef enum BeagleSpiSckSamplingEdge BeagleSpiSckSamplingEdge;
#endif

enum BeagleSpiBitorder {
    BG_SPI_BITORDER_MSB = 0,
    BG_SPI_BITORDER_LSB = 1
};
#ifndef __cplusplus
typedef enum BeagleSpiBitorder BeagleSpiBitorder;
#endif

int bg_spi_configure (
    Beagle                   beagle,
    BeagleSpiSSPolarity      ss_polarity,
    BeagleSpiSckSamplingEdge sck_sampling_edge,
    BeagleSpiBitorder        bitorder
);


int bg_spi_read (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    mosi_max_bytes,
    u08 *  data_mosi,
    int    miso_max_bytes,
    u08 *  data_miso
);


int bg_spi_read_data_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    mosi_max_bytes,
    u08 *  data_mosi,
    int    miso_max_bytes,
    u08 *  data_miso,
    int    max_timing,
    u32 *  data_timing
);


int bg_spi_read_bit_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    mosi_max_bytes,
    u08 *  data_mosi,
    int    miso_max_bytes,
    u08 *  data_miso,
    int    max_timing,
    u32 *  bit_timing
);



/*=========================================================================
| USB API
 ========================================================================*/
/* USB packet PID definitions */
#define BG_USB_PID_OUT 0xe1
#define BG_USB_PID_IN 0x69
#define BG_USB_PID_SOF 0xa5
#define BG_USB_PID_SETUP 0x2d
#define BG_USB_PID_DATA0 0xc3
#define BG_USB_PID_DATA1 0x4b
#define BG_USB_PID_DATA2 0x87
#define BG_USB_PID_MDATA 0x0f
#define BG_USB_PID_ACK 0xd2
#define BG_USB_PID_NAK 0x5a
#define BG_USB_PID_STALL 0x1e
#define BG_USB_PID_NYET 0x96
#define BG_USB_PID_PRE 0x3c
#define BG_USB_PID_ERR 0x3c
#define BG_USB_PID_SPLIT 0x78
#define BG_USB_PID_PING 0xb4
#define BG_USB_PID_RESERVE 0xf0
#define BG_READ_USB_ERR_BAD_SIGNALS 0x00010000
#define BG_READ_USB_ERR_BAD_SYNC 0x00020000
#define BG_READ_USB_ERR_BIT_STUFF 0x00040000
#define BG_READ_USB_ERR_FALSE_EOP 0x00080000
#define BG_READ_USB_ERR_LONG_EOP 0x00100000
#define BG_READ_USB_ERR_BAD_PID 0x00200000
#define BG_READ_USB_ERR_BAD_CRC 0x00400000
#define BG_READ_USB_HOST_DISCONNECT 0x00800000
#define BG_READ_USB_TARGET_DISCONNECT 0x01000000
#define BG_READ_USB_HOST_CONNECT 0x02000000
#define BG_READ_USB_TARGET_CONNECT 0x04000000
int bg_usb_read (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u08 *  packet
);


int bg_usb_read_data_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u08 *  packet,
    int    max_timing,
    u32 *  data_timing
);


int bg_usb_read_bit_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    int    max_bytes,
    u08 *  packet,
    int    max_timing,
    u32 *  bit_timing
);



/*=========================================================================
| MDIO API
 ========================================================================*/
enum BeagleMdioClause {
    BG_MDIO_CLAUSE_22    = 0,
    BG_MDIO_CLAUSE_45    = 1,
    BG_MDIO_CLAUSE_ERROR = 2
};
#ifndef __cplusplus
typedef enum BeagleMdioClause BeagleMdioClause;
#endif

#define BG_MDIO_OPCODE22_WRITE 0x01
#define BG_MDIO_OPCODE22_READ 0x02
#define BG_MDIO_OPCODE22_ERROR 0xff
#define BG_MDIO_OPCODE45_ADDR 0x00
#define BG_MDIO_OPCODE45_WRITE 0x01
#define BG_MDIO_OPCODE45_READ_POSTINC 0x02
#define BG_MDIO_OPCODE45_READ 0x03
/* Read the next MDIO frame. */
int bg_mdio_read (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    u32 *  data_in
);


int bg_mdio_read_bit_timing (
    Beagle beagle,
    u32 *  status,
    u64 *  time_sop,
    u64 *  time_duration,
    u32 *  time_dataoffset,
    u32 *  data_in,
    int    max_timing,
    u32 *  bit_timing
);


/*
 * Parse the raw MDIO data into the standard format.
 * This function will fill the supplied fields as per
 * the constants defined above.  If the raw data contains
 * a malformed turnaround field, the caller will be
 * notified of the error through the return value of
 * this function (BG_MDIO_BAD_TURNAROUND).
 */
int bg_mdio_parse (
    u32   packet,
    u08 * clause,
    u08 * opcode,
    u08 * addr1,
    u08 * addr2,
    u16 * data
);




#ifdef __cplusplus
}
#endif

#endif  /* __beagle_h__ */
