/*** COPYRIGHT SECTION ******************************************************
 *  bg_sniff.c
 *
 *  Contain funtion that is called to sniff ipmb packets from I2C bus.
 *  This code is copy from totalphase example with some modification.
 *  for Wireshark - Network traffic analyzer
 *
 *  Copyright (c) 2007 by KONTRON Canada Inc.
 *  chanthy.toeung@ca.kontron.com
 *  Base on example code from Totalphase Inc.
 *
 *  Wireshark - Network traffic analyzer
 *  By Gerald Combs <gerald@wireshark.org>
 *  Copyright 1999 Gerald Combs
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include "bg_sniff.h"



/*=========================================================================
| STATIC GLOBALS
 ========================================================================*/
static struct termios termattr, save_termattr;
static int ttysavefd = -1;

                 
/*=========================================================================
* TYPEDEFS
*=========================================================================*/

static enum 
{ 
  RESET, RAW, CBREAK 
} ttystate = RESET;


/*=========================================================================
| UTILITY FUNCTIONS
 ========================================================================*/
static void print_general_status (u32 status)
{
    printf(" ");

    /* General status codes*/
    if (status == BG_READ_OK)
        printf("OK ");

    if (status & BG_READ_TIMEOUT)
        printf("TIMEOUT ");

    if (status & BG_READ_ERR_MIDDLE_OF_PACKET)
        printf("MIDDLE ");
        
    if (status & BG_READ_ERR_SHORT_BUFFER)
        printf("SHORT BUFFER ");
   
    if (status & BG_READ_ERR_PARTIAL_LAST_BYTE)
        printf("PARTIAL_BYTE(bit %d) ", status & 0xff);
}

static void print_i2c_status (u32 status)
{
    /* I2C status codes*/
    if (status & BG_READ_I2C_NO_STOP)
        printf("I2C_NO_STOP ");

}

/*=========================================================================
| I2C DUMP FUNCTION
 ========================================================================*/
void i2cdump (int max_bytes, int num_packets, int num_device,
                Beagle beagle[MAXIMUM_PORT]) 
{
    /* Get the size of the timing information for a transaction of
    * max_bytes length
    */
    int timing_size = bg_bit_timing_size(BG_PROTOCOL_I2C, max_bytes);

    u16 *data_pointer;
    u16 *data_in = (u16 *)malloc(max_bytes*sizeof(u16));
    u32 *timing  = (u32 *)malloc(timing_size*sizeof(u32));
    int j;
    unsigned char ch;
    int leave = 0;
    u32 samplerate_khz[num_device];

    FILE* pFile;
    /* Empty the pFile b4 writing new entries */
    pFile = fopen( "rawData", "w" );
    if( pFile == NULL ){
        printf("error: can't write data into file\n");
        return;
    }
    /* Get the current sampling rate*/
    for (j = 0; j< num_device; j++){
      samplerate_khz[j] = bg_samplerate(beagle[j], 0);
    }
    int i, n;
    set_tty_raw(); /* set up character-at-a-time */
    /* Start the capture*/
    for( j =0; j< num_device; j++){
       if (bg_enable(beagle[j], BG_PROTOCOL_I2C) != BG_OK) {
           printf("error: could not enable I2C capture; exiting...\r\n");
           exit(1);
       }
    }
    printf("index,time(ns),I2C,status,<addr:r/w>(*),data0 ... dataN(*)\r\n");
    fflush(stdout);

    /* Capture and print each transaction*/
   for (i = 0; i < num_packets || !num_packets; ++i) {
      for( j = 0; j< num_device; j++){
        u32 status;
        u64 time_sop, time_sop_ns;
        u64 time_duration;
        u32 time_dataoffset;
        struct timeval start_time;
        /* Read transaction with bit timing data*/
        int count = bg_i2c_read_bit_timing(beagle[j], &status,
                                           &time_sop, &time_duration,
                                           &time_dataoffset,
                                           max_bytes, data_in,
                                           timing_size, timing);
        /* Get local time that it start to dump data*/
        gettimeofday( &start_time, NULL );

        /* Translate timestamp to ns*/
        time_sop_ns = TIMESTAMP_TO_NS(time_sop, samplerate_khz[j]);

        printf("%d," U64_FORMAT_STR ",I2C,(", i, time_sop_ns);

        if (count < 0)
            printf("error=%d,", count);
        data_in[count] = j;
        print_general_status(status);
        print_i2c_status(status);
        printf(")");

        /* Check for errors*/
        if (count <= 0) {
            printf("\r\n");
            fflush(stdout);

            if (count < 0)
                break;

            /* If zero data captured, continue*/
            continue;
        }

        /* Print the address and read/write*/
        printf(",");
        if (!(status & BG_READ_ERR_MIDDLE_OF_PACKET)) {
            /* Display the start condition*/
            printf("[S] ");

            if (count >= 1) {
                /* Determine if byte was NACKed*/
                int nack = (data_in[0] & BG_I2C_MONITOR_NACK);

                /* Determine if this is a 10-bit address*/
                if (count == 1 || (data_in[0] & 0xf9) != 0xf0 || nack) {
                    /* Display 7-bit address*/
                    printf("<%02x:%c>%s ", (data_in[0] & 0xff) >> 1,
                           data_in[0] & 0x01 ? 'r' : 'w',
                           nack ? "*" : "");
                    count--;
                    data_pointer = data_in + 1;
                }
                else {
                    /* Display 10-bit address*/
                    printf("<%03x:%c>%s ",
                           ((data_in[0] << 7) & 0x300) | (data_in[1] & 0xff),
                           data_in[0] & 0x01 ? 'r' : 'w',
                           nack ? "*" : "");
                    count -= 2;
                    data_pointer = data_in + 2;
                }
            }
        }

        /* Display rest of transaction*/
        for (n = 0; n < count; ++n) {
            /* Determine if byte was NACKed*/
            int nack = (*data_pointer & BG_I2C_MONITOR_NACK);

            printf("%02x%s ", *data_pointer & 0xff, nack ? "*" : "");
            ++data_pointer;
        }

        /* Display the stop condition*/
        if (!(status & BG_READ_I2C_NO_STOP))
            printf("[P] ==> port: %d", data_in[count+1]);
        
        printf("\r\n");
        /* start writing to File here */
        if( pFile != NULL && count > 0 ){
           int k = 0;
           count += 2;
           k = fwrite( &start_time, sizeof( start_time ), 1, pFile );
           if( !k )
               printf("error writing time.\r\n");  
           k = fwrite( &count, sizeof( count ), 1, pFile );   
           if( !k )
               printf("error writing count.\r\n");             
           k = fwrite( data_in, (size_t)count*sizeof(u16), 1, pFile );
           if( !k )
               printf("error writing data.\r\n");              
            printf("write packets to file %d\r\n", count );
        } /* end of writing to File */

        /* waiting for keyborad interrupt */

         ch = kb_getc();      /* char typed by user?*/
         if ( ch == 0x03 ){      /* might be control-C*/
            set_tty_cooked();  /* control-C, restore normal TTY mode*/ 
            leave = 1;
            break; 
         }
      }
      if( leave == 1)
         break;   
    }
    free(data_in);
    free(timing);
    fclose( pFile);
    /* Stop the capture*/
    for( j = 0; j<num_device; j++ ){
      bg_disable(beagle[j]);
    }
    printf("\rend of i2c dump\n");
}

/* ***************************************************************************
 *
 * set_tty_raw(), put the user's TTY in one-character-at-a-time mode.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int
set_tty_raw(void) 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
    printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }
  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
  termattr.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
  termattr.c_cflag &= ~(CSIZE | PARENB);
  termattr.c_cflag |= CS8;
  termattr.c_oflag &= ~(OPOST);
   
  termattr.c_cc[VMIN] = 1;  /* or 0 for some Unices;  see note 1 */
  termattr.c_cc[VTIME] = 0;

  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
    printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror("");
    return -1;
  }
   
  ttystate = RAW;
  ttysavefd = STDIN_FILENO;

  return 0;
}

/* ***************************************************************************
 *
 * set_tty_cbreak(), put the user's TTY in cbreak mode.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int 
set_tty_cbreak() 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
    printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }

  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON);
  termattr.c_cc[VMIN] = 1;
  termattr.c_cc[VTIME] = 0;
      
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
    printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }
  ttystate = CBREAK;
  ttysavefd = STDIN_FILENO;

  return 0;
}

/* ***************************************************************************
 *
 * set_tty_cooked(), restore normal TTY mode. Very important to call
 *   the function before exiting else the TTY won't be too usable.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int
set_tty_cooked() 
{
  int i;
  if (ttystate != CBREAK && ttystate != RAW) 
  {
    return 0;
  }
  i = tcsetattr (STDIN_FILENO, TCSAFLUSH, &save_termattr);
  if (i < 0) 
  {
    return -1;
  }
  ttystate = RESET;
  return 0;
}

/* ***************************************************************************
 *
 * kb_getc(), if there's a typed character waiting to be read,
 *   return it; else return 0.
 * 10-sep-2006: kb_getc() fails (it hangs on the read() and never returns
 * until a char is typed) under some Unix/Linux versions: ubuntu, suse, and
 * maybe others. To make it work, please uncomment two source lines below.
 *
 *************************************************************************** */
unsigned char
kb_getc(void) 
{
  int i;
  unsigned char ch;
  ssize_t size;
  termattr.c_cc[VMIN] = 0;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  size = read (STDIN_FILENO, &ch, 1);
  termattr.c_cc[VMIN] = 1;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (size == 0)
  {
    return 0;
  }
  else
  {
    return ch;
  }
}

/* ***************************************************************************
 *
 * kb_getc_w(), wait for a character to be typed and return it.
 *
 *************************************************************************** */
unsigned char
kb_getc_w(void) 
{
  unsigned char ch;
  size_t size;

  while (1)
  {

    usleep(20000);        /* 1/50th second: thanks, Floyd! */

    size = read (STDIN_FILENO, &ch, 1);
    if (size > 0)
    {
      break;
    }
  }
  return ch;
}





