/*** COPYRIGHT SECTION ******************************************************
 *  i2c_analyzer.c
 *
 *  Main program of the i2c analyzer tool to capturing raw data from i2c bus.
 *  for Wireshark - Network traffic analyzer
 *
 *  Copyright (c) 2007 by KONTRON Canada Inc.
 *  chanthy.toeung@ca.kontron.com
 *
 *  Wireshark - Network traffic analyzer
 *  By Gerald Combs <gerald@wireshark.org>
 *  Copyright 1999 Gerald Combs
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*=========================================================================
| INCLUDES
 ========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "beagle.h"
#include "bg_sniff.h"
#include "bg_ipmipacket.h"
#include "bg_hexdump.h"
#include "display.h"




/*=========================================================================
| USAGE INFORMATION
 ========================================================================*/
static void print_usage (void)
{
    printf(
"Usage: i2c_analyzer max_packet_len num_events port_number ...\n"
"Example utility for capturing I2C data from Beagle protocol analyzers.\n"
"\n"
"  The parameter max_packet_len is set to the maximum expected packet length\n"
"  throughout the entire capture session.\n"
"\n"                   
"  The parameter num_events is set to the number of events to process\n"
"  before exiting.  If num_events is set to zero, the capture will continue\n"
"  indefinitely.\n"
"\n"
"  The parameter port_number represents the port number that Beagle device is\n"
"  connected in the system. It start with 0 for the first port, 1 for the second"
"...\n\n"
"For product documentation and specifications, see www.totalphase.com.\n");
    fflush(stdout);
};


/*=========================================================================
| MAIN PROGRAM ENTRY POINT
| usage: i2c_analyzer [option] max_packet_len num_events port_number ...\n"
 ========================================================================*/
int main (int argc, char *argv[])
{
    int port[MAXIMUM_PORT];      /* open port 0 by default*/
    int samplerate = 10000;  /* in kHz*/
    int timeout    = 500;    /* in milliseconds*/
    int latency    = 200;    /* in milliseconds*/
    int len        = 0;
    int pullups    = BG_I2C_PULLUP_OFF;
    int target_pow = BG_TARGET_POWER_OFF;
    int num        = 0;
    int i = 0;
    int numberOfDevice = 1;
    Beagle beagle[MAXIMUM_PORT];
    if (argc < 4)
    {
        print_usage();
	     exit(1);
    }
    else{
      normalProcess = get_option( argc, argv );
      quiet = get_quiet_option();
      debug = get_debug_level();
      withOption = get_opt();
    }
    if( normalProcess ){
      /* without option, the second argument will be the filename */
      if( !withOption ){
         len = atoi(argv[1]);
         num = atoi(argv[2]);
         argc -= 3;
         for( i = 0; i<argc; i++){
            port[i] = atoi(argv[i+3]);
         }
      }
      else{
         if( quiet ){
            len = atoi(argv[2]);
            num = atoi(argv[3]);
            argc -= 4;
            for( i = 0; i<argc; i++){
               port[i] = atoi(argv[i+4]);
            }
         }
         else{
            len = atoi(argv[3]);
            num = atoi(argv[4]);
            argc -= 5;
            for( i = 0; i<argc; i++){
               port[i] = atoi(argv[i+5]);
            }
         }
      }
       numberOfDevice = argc;
       printf("num of device found = %d\n", numberOfDevice);

       /* Open the device*/
       for (i = 0; i<numberOfDevice; i++){
         beagle[i] = bg_open(port[i]);   
         if (beagle[i] <= 0) {
           printf("Unable to open Beagle device on port %d\n", port[i]);
           printf("Error code = %d\n", beagle[i]);
           return 1;
          }
          printf("Opened Beagle device on port %d\n", port[i]);
          /* Set the samplerate*/
          samplerate = bg_samplerate(beagle[i], samplerate);
          if (samplerate < 0) {
              printf("error: %s\n", bg_status_string(samplerate));
              return 1;
          }
          printf("Sampling rate set to %d KHz.\n", samplerate);
      
          /* Set the idle timeout.
          * The Beagle read functions will return in the specified time
          * if there is no data available on the bus.*/
          bg_timeout(beagle[i], timeout);
          printf("Idle timeout set to %d ms.\n", timeout);
      
          /* Set the latency.
          * The latency parameter allows the programmer to balance the
          * tradeoff between host side buffering and the latency to
          * receive a packet when calling one of the Beagle read
          * functions.*/
          bg_latency(beagle[i], latency);
          printf("Latency set to %d ms.\n", latency);
      
          printf("Host interface is %s.\n",
                 (bg_host_ifce_speed(beagle[i])) ? "high speed" : "full speed");
      
          /* There is usually no need for pullups or target power
          * when using the Beagle as a passive monitor.*/
          bg_i2c_pullup(beagle[i], (u08)pullups);
          bg_target_power(beagle[i], (u08)target_pow);
       }
      }    
    printf("\n");
    fflush(stdout);

    i2cdump(len, num, numberOfDevice, beagle);
    /* build ipmb packet */
    fipmipacket( "rawData" );
    /* build ipmb matching packet */
    fmatching_packet( "matching.txt" );
    /* create hexdump file of the receiving packets */
    hexdump_file( "matching.txt", "hexdump_bg" ); 
    /* Close the device*/
    for( i=0; i< numberOfDevice; i++ ){
      bg_close(beagle[i]);
    }

    return 0;
}
